# Deep Image Prior for Single-Shot, Blind Denoising and Deconvolution ##
### EE 367 Final Project ###
#### By Rohan Sanda (rsanda@stanford.edu) and Akhilesh Balasingam (avb03@stanford.edu) ####

#### Description ####
Denoising and deconvolution are fundamental inverse problems in image processing and computer vision, addressing issues arising from noise and blur in images. In this paper, we investigate untrained methods for solving these inverse problems, particularly blind denoising and deblurring, where prior data is unavailable. Our study explores classical methods, supervised methods, and unsupervised methods. For unsupervised methods, we implement and evaluate the Deep Image Prior (DIP) against several classical and trained baselines [1]. Additionally, we propose adaptations to the DIP framework for blind denoising and deconvolution, utilizing TV regularization and secondary neural networks as additional priors for image reconstruction [2], [3]. We experiment with different architectures and regularization techniques, evaluating their performance using standard metrics such as PSNR, SSIM, and DSSIM. Results show promising outcomes for DIP-based approaches, especially when combined with additional regularization such as total variation (TV).


#### Running ####
To run our code, first download the BSDS300 dataset and move the folder into `src/`. Then, simply set your configurations for each model you would like to run in `config.py.` By default, we run the denoising task on the BM3D, DIP, and DIP+TV models. Then run `python run.py` to run the models. Edit `run.py` to run additional models. Output data will be stored in `src/results/`. 

#### Citations ####
[1] D. Ulyanov, A. Vedaldi, and V. Lempitsky, “Deep image prior,” International Journal of Computer Vision vol. 128, no. 7, p. 867–1888, Mar. 2020. [Online]. Available: http://dx.doi.org/10.1007/s11263-020-01303-4

[2] D. Ren, K. Zhang, Q. Wang, Q. Hu, and W. Zuo, “Neural blind deconvolution using deep priors,” 2020

[3] CP. Cascarano, A. Sebastiani, M. C. Comes, G. Franchini, and F. Porta, “Combining weighted total variation and deep image prior for natural and medical image restoration via admm,” in 2021 21st International Conference on Computational Science and Its Applications (ICCSA). IEEE, Sep. 2021. [Online]. Available: http://dx.doi.org/10.1109/ICCSA54496.2021.00016




 


