###################################################
#   Solution code for EE367 HW6, task 3
#
#   Gordon Wetzstein, 10/2021
###################################################

import numpy as np
from scipy.sparse.linalg import cg, LinearOperator
from tqdm import tqdm
from itertools import product
from tqdm import tqdm

def leastnorm(b, Afun, Atfun, num_iters, imageResolution):

    # number of measurements
    N = b.shape[0]

    # convergence tolerance of cg solver
    cg_tolerance = 1e-12

    # initialize x with all zeros
    x = np.zeros(imageResolution)

    ################# begin task 3 ###################################
    #   Your task: implement a matrix-free conjugate gradient solver 
    #       using the scipy.sparse.linalg.cg function in combination 
    #       with the provided function handles Afun and Atfun. Use 
    #       num_iters as the number of iterations of cg and 
    #       cg_tolerance as the "tol" parameters for the cg function.
    #   
    #   Hints: 
    #       1. solve the problem (AA')y = b using CG first (CG only 
    #           works for positive semi-definite matrices, like AA')
    #       2. then multiply the result by A' to get x as x=A'y
    #       
    #   Be careful with your image dimensions. The cg function expects
    #   vector inputs and outputs, whereas b, x, Afun, Atfun all
    #   work with 2D images. So make sure you work with the vectorized
    #   versions for the function handles you pass into cg and then reshape
    #   the result to a 2D image again after.
    
    
    def A(x):
        return Afun(x.reshape(-1)).reshape(imageResolution)

    def At(x):
        return Atfun(x.reshape(-1)).reshape(imageResolution)

    # Linear operator for the matrix AA'
    AA_operator = LinearOperator((N, N), matvec=lambda x : Afun(Atfun(x)))

    # Solve the system (AA')y = b using CG
    y, _ = cg(AA_operator, b.ravel(), tol=cg_tolerance, maxiter=num_iters)

    # Calculate x = A'y
    x = At(y)

    ################# end task 3 ###################################

    return x



    