"""
Rohan Sanda, Akhilesh Balasingam 2024

This is the main run script. Modify this with the methods you want to run (see below).
"""

import torch
import torch.nn as nn
from torch.utils.data import Dataset, DataLoader
from glob import glob
import os
from skimage.metrics import peak_signal_noise_ratio as compare_psnr
import skimage.io
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
from tqdm import tqdm
import pickle

# set random seeds
torch.manual_seed(1)
np.random.seed(1)

torch.use_deterministic_algorithms(True)

matplotlib.rcParams['figure.raise_window'] = False

device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

from config import *
from models_dip import *
from utils import *
from denoise_deblur_dip import *
from hw5_task2 import run_hw5
from denoise_dip_tv import admm_dip_single
from bm3d_wrapper import bm3d_single
from hw5_task2 import BSDS300Dataset, MRIDataset
from hw5_task3 import BlurredBSDS300Dataset
from run_options import *
from PIL import Image
import torchvision.transforms.functional as TF
import torchvision.transforms as T

# Get evaluation data
dataset = BSDS300Dataset(patch_size=32, split='test', use_patches=False)
subset = get_random_subset(dataset, subset_size=5)
processed_pils, processed_tensors = process_subset(subset)
n = len(processed_pils)

# Which methods to run
run_method(subset, dataset_name="BSDS300", task="denoise", method= "DCNN", fsavepath="./results", verbose=False)
run_method(subset, dataset_name="BSDS300", task="deblur", method= "DIP", fsavepath="./results", verbose=False)
run_method(subset, dataset_name="BSDS300", task="denoise", method= "ADMM-DIP", fsavepath="./results", verbose=False)

